package com.example.core.component.input

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.animateColorAsState
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.BasicTextField
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.Icon
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.focus.focusRequester
import androidx.compose.ui.focus.onFocusChanged
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.SolidColor
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.text.input.VisualTransformation
import androidx.compose.ui.unit.dp
import com.example.core.theme.CustomIcons
import com.example.core.theme.CustomTheme

/**
 * Автор: Манякин Дмитрий (user5)
 * Дата создания: 26.05.2025
 * */

@Composable
fun EnterInputField(
    value: String,
    onValueChange: (String) -> Unit,
    label: String, // названия поля (надпись над полем)
    hint: String, // подсказка
    errorMessage: String, // сообщенме об ошибке
    modifier: Modifier = Modifier,
    isPassword: Boolean = false, // являяется ли паролем (для отображения и скрытия)
    keyboardOptions: KeyboardOptions = KeyboardOptions.Default // смена типа клавиатуры (только цифры и тд)
) {

    /**
     * отслеживание состояния фокуса для обводки активного поля
     * */
    val focusRequester = remember { FocusRequester() }

    var hasFocus by rememberSaveable { mutableStateOf(false) }

    var showPassword by rememberSaveable { mutableStateOf(false) }

    /**
     * Установка цвета рамки в зависимости от состояния
     * */
    val borderColor by animateColorAsState(
        when {
            errorMessage.isNotBlank() -> CustomTheme.colors.error
            errorMessage.isBlank() && hasFocus -> CustomTheme.colors.accent
            value.isEmpty() || (value.isNotEmpty() && label.isNotEmpty()) || isPassword -> CustomTheme.colors.inputStroke
            else -> CustomTheme.colors.inputIcon
        }
    )

    /**
     * установка цвета поверх конентка в случае ошибки
    * */
    val overlayErrorColor by animateColorAsState(
        if (errorMessage.isNotBlank()) CustomTheme.colors.error.copy(alpha = 0.1f) else Color.Transparent
    )

    Column(modifier = modifier) {
        if (label.isNotBlank()) {
            Text(
                text = label,
                style = CustomTheme.typography.captionRegular.copy(
                    color = CustomTheme.colors.description
                )
            )

            Spacer(Modifier.height(CustomTheme.elevation.spacing8dp))
        }

        Box(
            contentAlignment = Alignment.CenterStart,
            modifier = Modifier
                .fillMaxWidth()
                .height(CustomTheme.elevation.spacing48dp)
                .clip(RoundedCornerShape(10.dp))
                .background(CustomTheme.colors.inputBg)
                .border(width = 1.dp, color = borderColor, shape = RoundedCornerShape(10.dp))
        ) {
            Box(
                modifier = Modifier
                    .fillMaxSize()
                    .background(overlayErrorColor)
            )

            Row(modifier = Modifier.fillMaxSize(), verticalAlignment = Alignment.CenterVertically) {
                Spacer(Modifier.width(14.dp))

                Box(contentAlignment = Alignment.CenterStart, modifier = Modifier.weight(1f)) {

                    if (value.isEmpty()) {
                        Text(
                            text = hint,
                            style = CustomTheme.typography.textRegular.copy(
                                color = CustomTheme.colors.placeholder
                            )
                        )
                    }

                    BasicTextField(
                        value = value,
                        onValueChange = onValueChange,
                        singleLine = true,
                        cursorBrush = SolidColor(CustomTheme.colors.accent),
                        textStyle = CustomTheme.typography.textRegular,
                        keyboardOptions = keyboardOptions,
                        visualTransformation = if (isPassword && !showPassword) {
                            PasswordVisualTransformation()
                        } else VisualTransformation.None,
                        modifier = Modifier
                            .focusRequester(focusRequester)
                            .onFocusChanged { focusState ->
                                hasFocus = focusState.hasFocus
                            }
                    )
                }

                if (isPassword) {
                    val icon =
                        if (showPassword) CustomIcons.showPassword else CustomIcons.hidePassword
                    Icon(
                        painter = icon,
                        contentDescription = null,
                        modifier = Modifier.clickable(
                            interactionSource = remember { MutableInteractionSource() },
                            indication = null
                        ) { showPassword = !showPassword }
                    )
                    Spacer(Modifier.width(15.dp))
                }
            }
        }

        /**
         * отображение текста ошибки в случае ее наличия
        * */
        AnimatedVisibility(visible = errorMessage.isNotBlank()) {
            Column {
                Spacer(Modifier.height(CustomTheme.elevation.spacing8dp))
                Text(
                    text = errorMessage,
                    style = CustomTheme.typography.captionRegular.copy(
                        color = CustomTheme.colors.error
                    )
                )
            }
        }
    }
}