package com.example.core.component.select

import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.heightIn
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.Text
import androidx.compose.material3.rememberModalBottomSheetState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.unit.dp
import com.example.core.theme.CustomIcons
import com.example.core.theme.CustomTheme
import kotlinx.coroutines.launch

/**
 * Автор: Манякин Дмитрий (user5)
 * Дата создания: 26.05.2025
 * */

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun AppSelector(
    selectedOption: Option?, // выбранная опция, если null то отображается подсказка
    options: List<Option>, // список опций
    onOptionSelect: (Option) -> Unit, // обработка нажатия
    hint: String, // подсказка
    modifier: Modifier = Modifier
) {
    var showSheet by rememberSaveable { mutableStateOf(false) }
    val sheetState = rememberModalBottomSheetState()
    val scope = rememberCoroutineScope()

    if (showSheet) {
        AppBottomSheet(
            sheetState = sheetState,
            onDismissRequest = { showSheet = false },
            modifier = Modifier.testTag("AppBottomSheet")
        ) {
            Column(verticalArrangement = Arrangement.Center) {
                options.forEach { option ->
                    Row(
                        modifier = Modifier
                            .heightIn(min = 40.dp)
                            .fillMaxWidth()
                            .clickable(
                                remember { MutableInteractionSource() },
                                null
                            ) {
                                scope.launch {
                                    onOptionSelect(option)
                                    sheetState.hide()
                                }.invokeOnCompletion {
                                    if (!sheetState.isVisible) {
                                        showSheet = false
                                    }
                                }
                            }
                    ) {
                        Text(
                            text = option.label,
                            style = CustomTheme.typography.title2ExtraBold
                        )
                    }
                }
            }
        }
    }

    Column(modifier = modifier) {
        Box(
            contentAlignment = Alignment.CenterStart,
            modifier = Modifier
                .fillMaxWidth()
                .height(CustomTheme.elevation.spacing48dp)
                .clip(RoundedCornerShape(10.dp))
                .background(CustomTheme.colors.inputBg)
                .border(
                    width = 1.dp,
                    color = CustomTheme.colors.inputStroke,
                    shape = RoundedCornerShape(10.dp)
                )
        ) {
            Row(
                modifier = Modifier
                    .fillMaxSize()
                    .clickable(
                        interactionSource = remember { MutableInteractionSource() },
                        indication = null
                    ) { showSheet = true },
                verticalAlignment = Alignment.CenterVertically
            )
            {
                Spacer(Modifier.width(14.dp))

                Box(contentAlignment = Alignment.CenterStart, modifier = Modifier.weight(1f)) {
                    selectedOption?.let {
                        Text(
                            text = selectedOption.label,
                            style = CustomTheme.typography.headlineRegular
                        )
                    } ?: run {
                        Text(
                            text = hint,
                            style = CustomTheme.typography.textRegular.copy(
                                color = CustomTheme.colors.placeholder
                            )
                        )
                    }
                }

                Icon(
                    painter = CustomIcons.more,
                    tint = CustomTheme.colors.description,
                    contentDescription = null
                )
                Spacer(Modifier.width(14.dp))
            }
        }
    }
}